@@ -7,16 +7,15 @@ Copyright (C) 2019 Martin Fleischmann, 2017 Nyall Dawson
 
 import operator
 
-__all__ = [
-    "greedy",
-]
+__all__ = ["greedy"]
 
 
 def _balanced(features, sw, balance="count", min_colors=4):
     """
     Strategy to color features in a way which is visually balanced.
 
-    Algorithm ported from QGIS to be used with GeoDataFrames and libpysal weights objects.
+    Algorithm ported from QGIS to be used with GeoDataFrames
+    and libpysal weights objects.
 
     Original algorithm:
     Date                 : February 2017
@@ -25,19 +24,22 @@ def _balanced(features, sw, balance="count", min_colors=4):
 
     Parameters
     ----------
-    features : GeoDataFrame
-        GeoDataFrame
+
+    features : geopandas.GeoDataFrame
+        GeoDataFrame.
     sw : libpysal.weights.W
-        spatial weights object denoting adjacency of features
-    balance : str
-        the method of color balancing
-    min_colors : int
-        the minimal number of colors to be used
+        Spatial weights object denoting adjacency of features.
+    balance : str (default 'count')
+        The method of color balancing.
+    min_colors : int (default 4)
+        The minimal number of colors to be used.
 
     Returns
     -------
+
     feature_colors : dict
-        dictionary with assigned color codes
+        Dictionary with assigned color codes.
+
     """
     feature_colors = {}
     # start with minimum number of colors in pool
@@ -46,7 +48,7 @@ def _balanced(features, sw, balance="count", min_colors=4):
     # calculate count of neighbours
     neighbour_count = sw.cardinalities
 
-    # sort features by neighbour count - we want to handle those with more neighbours first
+    # sort features by neighbour count - handle those with more neighbours first
     sorted_by_count = [
         feature_id
         for feature_id in sorted(
@@ -77,7 +79,7 @@ def _balanced(features, sw, balance="count", min_colors=4):
 
         feature_color = -1
         if len(available_colors) == 0:
-            # no existing colors available for this feature, so add new color to pool and repeat
+            # no existing colors available for this feature; add new color and repeat
             min_colors += 1
             return _balanced(features, sw, balance, min_colors)
         else:
@@ -107,7 +109,7 @@ def _balanced(features, sw, balance="count", min_colors=4):
                 }
 
                 distances = features.loc[other_features.keys()].distance(this_feature)
-                # loop through these, and calculate the minimum distance from this feature to the nearest
+                # calculate the min distance from this feature to the nearest
                 # feature with each assigned color
                 for other_feature_id, c in other_features.items():
 
@@ -115,8 +117,8 @@ def _balanced(features, sw, balance="count", min_colors=4):
                     if distance < min_distances[c]:
                         min_distances[c] = distance
 
-                # choose color such that minimum distance is maximised! ie we want MAXIMAL separation between
-                # features with the same color
+                # choose color such that min distance is maximised!
+                # - ie we want MAXIMAL separation between features with the same color
                 feature_color = sorted(
                     min_distances, key=min_distances.__getitem__, reverse=True
                 )[0]
@@ -126,55 +128,6 @@ def _balanced(features, sw, balance="count", min_colors=4):
     return feature_colors
 
 
-def _geos_sw(features, tolerance=0, silence_warnings=False, resolution=5):
-    """
-    Generate libpysal spatial weights object based on intersections of features.
-
-    Intersecting features are denoted as neighbours. If tolerance > 0, all features
-    within the set tolerance are denoted as neighbours.
-
-
-    Parameters
-    ----------
-    features : GeoDataFrame
-        GeoDataFrame
-    tolerance : float (default 0)
-        minimal distance between colors
-    silence_warnings : bool (default True)
-        silence lilbpysal warnings (if min_distance is set)
-    resolution : int (default 5)
-        resolution of buffer if tolerance > 0
-
-    Returns
-    -------
-    W : libpysal.weights.W
-        spatial weights object
-    """
-    try:
-        from libpysal.weights import W
-    except ImportError:
-        raise ImportError("The 'libpysal' package is required.")
-
-    neighbors = {}
-
-    if tolerance > 0:
-        features = features.copy()
-        features["geometry"] = features.geometry.buffer(tolerance / 2, resolution)
-
-    sindex = features.sindex
-
-    for i, (ix, g) in enumerate(features.geometry.iteritems()):
-
-        possible_matches_index = list(sindex.intersection(g.bounds))
-        possible_matches_index.remove(i)
-        possible_matches = features.iloc[possible_matches_index]
-        precise_matches = possible_matches.loc[possible_matches.intersects(g)]
-
-        neighbors[ix] = list(precise_matches.index)
-
-    return W(neighbors, silence_warnings=silence_warnings)
-
-
 def greedy(
     gdf,
     strategy="balanced",
@@ -190,20 +143,20 @@ def greedy(
 
     Attempts to color a GeoDataFrame using as few colors as possible, where no
     neighbours can have same color as the feature itself. Offers various strategies
-    ported from QGIS or implemented within networkX for greedy graph coloring.
+    ported from QGIS or implemented within NetworkX for greedy graph coloring.
 
-    ``greedy`` will return pandas.Series representing assinged color codes.
+    ``greedy`` will return ``pandas.Series`` representing assigned color codes.
 
     Parameters
     ----------
+
     gdf : GeoDataFrame
         GeoDataFrame
     strategy : str (default 'balanced')
-        Determine coloring strategy. Options are ``'balanced'`` for algorithm based on
-        QGIS Topological coloring. It is aiming for a visual balance, defined by the
-        balance parameter.
-
-        Other options are those supported by networkx.greedy_color:
+        Determine coloring strategy. Options are ``'balanced'`` for
+        algorithm based on QGIS Topological coloring. It is aiming
+        for a visual balance, defined by the balance parameter. Other
+        options are those supported by ``networkx.greedy_color``:
 
         * ``'largest_first'``
         * ``'random_sequential'``
@@ -215,75 +168,111 @@ def greedy(
         * ``'saturation_largest_first'``
         * ``'DSATUR'`` (alias for the previous strategy)
 
-        For details see
-        https://networkx.github.io/documentation/stable/reference/algorithms/generated/networkx.algorithms.coloring.greedy_color.html
+        For details see https://networkx.github.io/documentation/stable/reference/algorithms/generated/networkx.algorithms.coloring.greedy_color.html
 
     balance : str (default 'count')
         If strategy is ``'balanced'``, determine the method of color balancing.
 
         * ``'count'`` attempts to balance the number of features per each color.
         * ``'area'`` attempts to balance the area covered by each color.
-        * ``'centroid'`` attempts to balance the distance between colors based on the distance between centroids.
-        * ``'distance'`` attempts to balance the distance between colors based on the distance between geometries. Slower than ``'centroid'``, but more precise.
-
-        ``'centroid'`` and ``'distance'`` are significantly slower than other especially
-        for larger GeoDataFrames.
-
-        Apart from ``'count'``, all require CRS to be projected (not in degrees) to ensure
-        metric values are correct.
+        * ``'centroid'`` attempts to balance the distance between colors based
+          on the distance between centroids.
+        * ``'distance'`` attempts to balance the distance between colors based
+          on the distance between geometries. Slower than ``'centroid'``,
+          but more precise.
 
+        Both ``'centroid'`` and ``'distance'`` are significantly slower than other
+        especially for larger GeoDataFrames. Apart from ``'count'``, all require
+        CRS to be projected (not in degrees) to ensure metric values are correct.
     min_colors: int (default 4)
         If strategy is ``'balanced'``, define the minimal number of colors to be used.
-
     sw : 'queen', 'rook' or libpysal.weights.W (default 'queen')
-        If min_distance is None, one can pass ``'libpysal.weights.W'`` object denoting neighbors
-        or let greedy to generate one based on ``'queen'`` or ``'rook'`` contiguity.
-
-    min_distance : float
-        Set minimal distance between colors.
-
-        If min_distance is not None, slower algorithm for generating spatial weghts is used
-        based on intersection between geometries. Min_distance is then used as a tolerance
-        of intersection.
-
+        If min_distance is None, one can pass ``'libpysal.weights.W'``
+        object denoting neighbors or let greedy generate one based on
+        ``'queen'`` or ``'rook'`` contiguity.
+    min_distance : float (default None)
+        Set minimal distance between colors. If ``min_distance`` is not ``None``,
+        slower algorithm for generating spatial weghts is used based on
+        intersection between geometries. ``'min_distance'`` is then used as a
+        tolerance of intersection.
     silence_warnings : bool (default True)
         Silence libpysal warnings when creating spatial weights.
-
-    interchange : bool (defaul False)
-        Use the color interchange algorithm (applicable for networkx strategies)
-
-        For details see
-        https://networkx.github.io/documentation/stable/reference/algorithms/generated/networkx.algorithms.coloring.greedy_color.html
-
-    Examples
-    --------
-    Default:
-
-    >>> gdf['greedy_colors'] = greedy(gdf)
-
-    Balanced by area:
-
-    >>> gdf['balanced_area'] = greedy(gdf, strategy='balanced',
-    >>>                               balance='area')
-
-    Using rook adjacency:
-
-    >>> gdf['rook_adjacency'] = greedy(gdf, sw='rook')
-
-    Adding minimal distance between colors:
-
-    >>> gdf['min_distance'] = greedy(gdf, min_distance=100)
-
-    Using different coloring strategy:
-
-    >>> gdf['smallest_last'] = greedy(gdf, strategy='smallest_last')
-
+    interchange : bool (default False)
+        Use the color interchange algorithm (applicable for NetworkX strategies).
+        For details see https://networkx.github.io/documentation/stable/reference/algorithms/generated/networkx.algorithms.coloring.greedy_color.html
 
     Returns
     -------
-    color : pd.Series
-        pandas.Series representing assinged color codes
-    """
+
+    color : pandas.Series
+        ``pandas.Series`` representing assinged color codes.
+
+    Examples
+    --------
+
+    >>> from mapclassify import greedy
+    >>> import geopandas
+    >>> world = geopandas.read_file(geopandas.datasets.get_path("naturalearth_lowres"))
+    >>> africa = world.loc[world.continent == "Africa"].copy()
+    >>> africa = africa.to_crs("ESRI:102022").reset_index(drop=True)
+
+    Default:
+
+    >>> africa["greedy_colors"] = greedy(africa)
+    >>> africa["greedy_colors"].head()
+    0    1
+    1    0
+    2    0
+    3    1
+    4    4
+    Name: greedy_colors, dtype: int64
+
+    Balanced by area:
+
+    >>> africa["balanced_area"] = greedy(africa, strategy="balanced", balance="area")
+    >>> africa["balanced_area"].head()
+    0    1
+    1    2
+    2    0
+    3    1
+    4    3
+    Name: balanced_area, dtype: int64
+
+    Using rook adjacency:
+
+    >>> africa["rook_adjacency"] = greedy(africa, sw="rook")
+    >>> africa["rook_adjacency"].tail()
+    46    3
+    47    0
+    48    2
+    49    3
+    50    1
+    Name: rook_adjacency, dtype: int64
+
+    Adding minimal distance between colors:
+
+    >>> africa["min_distance"] = greedy(africa, min_distance=1000000)
+    >>> africa["min_distance"].head()
+    0    1
+    1    9
+    2    0
+    3    7
+    4    4
+    Name: min_distance, dtype: int64
+
+    Using different coloring strategy:
+
+    >>> africa["smallest_last"] = greedy(africa, strategy="smallest_last")
+    >>> africa["smallest_last"].head()
+    0    3
+    1    1
+    2    1
+    3    3
+    4    1
+    Name: smallest_last, dtype: int64
+
+    """  # noqa
+
     if strategy != "balanced":
         try:
             import networkx as nx
@@ -298,13 +287,18 @@ def greedy(
     except ImportError:
         raise ImportError("The 'pandas' package is required.")
     try:
-        from libpysal.weights import Queen, Rook, W
+        from libpysal.weights import Queen, Rook, W, fuzzy_contiguity
     except ImportError:
         raise ImportError("The 'libpysal' package is required.")
 
     if min_distance is not None:
-        # TODO: use libpysal's fuzzy_contiguity instead of _geos_sw once pysal/libpysal#280 is released
-        sw = _geos_sw(gdf, tolerance=min_distance, silence_warnings=silence_warnings)
+        sw = fuzzy_contiguity(
+            gdf,
+            tolerance=0.0,
+            buffering=True,
+            buffer=min_distance / 2.0,
+            silence_warnings=silence_warnings,
+        )
 
     if not isinstance(sw, W):
         if sw == "queen":
@@ -321,7 +315,7 @@ def greedy(
         )
 
     else:
-        raise ValueError("{} is not a valid strategy.".format(strategy))
+        raise ValueError(f"'{strategy}' is not a valid strategy.")
 
     color = pd.Series(color).sort_index()
     color.index = gdf.index
